'use strict';

let oz_wrapper_version = '20220901 18:10 - calling the callback fn in bidsbackhandler';
var debugDiv = document.createElement('div'); // this will be replaced if we find pbjs_debug=true

console.log('Ozone INCLUDE JS PAGE: ' + oz_wrapper_version);

var startTime = Date.now();
var pbjs = pbjs || {};
pbjs.que = pbjs.que || [];

var configVars = {
    adunitsFile_src: 'https://prebid.the-ozone-project.com/hw2/OZONEATV9207/3500000306/current/adUnits.min.js',
    ozonePrebidFile_src: 'https://prebid.the-ozone-project.com/hw2/OZONEATV9207/prebid.js',
    pbjsDebug: document.location.search.match('pbjs_debug=true'),
    prebidSingleRequest: true,
    callbackArg: 'ozone' /* this is what will be sent to the callback function in the bidsbackhandler */
}

// Debugging : we will write out console log info when GET param pbjs_debug=true exists.
function ozoneLog() {
    if(!configVars.pbjsDebug) return;
    let txt = arguments[0];
    if(arguments.length > 1) {
        txt = arguments;
    }
    console.log(`%cOZONE wrapper : ${Date.now() - startTime}ms`, 'background-color: #AFA; border-radius:5px', txt);
}
function ozoneError() {
    if(!configVars.pbjsDebug) return;
    let txt = arguments[0];
    if(arguments.length > 1) {
        txt = arguments;
    }
    console.error(`%cOZONE wrapper : ${Date.now() - startTime}ms`, 'background-color: #FAA; border-radius:5px', txt);
}

ozoneLog(configVars);

window.googletag = window.googletag || {};
window.googletag.cmd = window.googletag.cmd || [];

// These lines don't need to be added directly in the html page so long as this file is included early,
// and synchronously (before any googletag.defineSlot calls are made)
var ozoneadunits = ozoneadunits || {};
ozoneadunits.cmd = ozoneadunits.cmd || [];

// if this is pbjs_debug=true then write out the version on the screen to confirm you've got your redirect correct
if(configVars.pbjsDebug) {
    ozoneadunits.cmd.push(function () {
        debugDiv = document.createElement('div');
        debugDiv.style = "position:fixed; top:200px;right:0px;width: 300px; height: 80px; background-color: yellow; z-index: 99999; padding: 10px; font-size:10pt; line-height: 12px";
        debugDiv.innerText = `OZ wrapper: ${oz_wrapper_version}`;
        document.body.appendChild(debugDiv);
    });
}

// include adunits file asynchronously so we don't delay execution unduly
var adunitsFile = document.createElement('script');
adunitsFile.src = configVars.adunitsFile_src;
document.head.appendChild(adunitsFile); // using appendChild should cause this to load asynchronously
ozoneLog('pbjs.js: after writing script tag to include adunits.js');

var prebidFile = document.createElement('script');
prebidFile.src = configVars.ozonePrebidFile_src;
document.head.appendChild(prebidFile);
ozoneLog('pbjs.js: after writing script tag to include prebid.js');

var customPriceGranularityChoices = {
    "low": "low",
    "medium": "medium",
    "high": "high",
    "auto": "auto",
    "dense": "dense",
    "custom": {
        "buckets": [{"increment":0.01,"min":0,"max":2.5,"precision":2},{"increment":0.02,"min":2.5,"max":5,"precision":2},{"increment":0.05,"min":5,"max":7.5,"precision":2},{"increment":0.1,"min":7.5,"max":10,"precision":2},{"increment":0.2,"min":10,"max":15,"precision":2},{"increment":0.5,"min":15,"max":27.5,"precision":2}]
    }
};

pbjs.que.push(function () {
    pbjs.setConfig({
        
        
        userSync: {
            userIds: [
                {
                    name: "sharedId",
                    storage: {
                        type: "html5",
                        name: "_sharedID",
                        expires: 365
                    }
                }
            ],
            filterSettings: {
                iframe: {
                    bidders: '*',
                    filter: 'include',
                    iframeEnabled: true
                }
            },
            syncDelay: 3000
        },

        ozone: {
            singleRequest: true,
            enhancedAdserverTargeting: true,
            

            
        },

        consentManagement: {
 "gdpr": {
  "cmpApi": "iab",
  "allowAuctionWithoutConsent": true,
  "timeout": 10000,
  "defaultGdprScope": true
 }
},

        sizeConfig:[
 {
  "mediaQuery": "(min-width: 0px) and (max-width: 375px)",
  "sizesSupported": [
   [
    300,
    250
   ],
   [
    320,
    50
   ]
  ],
  "labels": [
   "mobile"
  ]
 },
 {
  "mediaQuery": "(min-width: 376px) and (max-width: 768px)",
  "sizesSupported": [
   [
    728,
    90
   ],
   [
    300,
    250
   ],
   [
    970,
    250
   ]
  ],
  "labels": [
   "tablet"
  ]
 },
 {
  "mediaQuery": "(min-width: 769px) and (max-width: 10260px)",
  "sizesSupported": [
   [
    970,
    250
   ],
   [
    728,
    90
   ],
   [
    300,
    600
   ],
   [
    300,
    250
   ]
  ],
  "labels": [
   "desktop"
  ]
 }
],
        
        enableSendAllBids: true, 
        useBidCache: false,

        priceGranularity: customPriceGranularityChoices['custom'],

    });

});

var ozoneWrapper = {

    // note there is no need for timeouts - we are NOT going to call the adserver at all!!

    /**
     * 20220812 - when individual ads are refreshed, they are expecting to refresh the adunit themselves - we must not refresh
     *
     * This is the only function that publishers have to call.
     * NOTE this now REQUIRES 2 args to be sent.
     *
     * @param Array|null optionalDivIds - send an array of divIDs if you want to restrict to only these, else null & we will work on all adunits known to googletag
     * @param Callable callback
     * @return {boolean}
     */
    ozoneFetchBids: function(optionalDivIds, callback) {
        ozoneLog(`ozoneFetchBids was called. document.readyState is ${document.readyState} and optionalDivIds: `, optionalDivIds);
        if(document.readyState === 'loading') return false;

        // new code - optionally restrict to a set of div ids
        let arrDivids = [];
        if(arguments.length !== 2 ) {
            ozoneError(`ozoneFetchBids needs 2 args (optional div IDs|null, callable callback fn). ${arguments.length} args were sent.`);
            return;
        }
        if(Array.isArray(optionalDivIds)) {
            // ozoneLog('ozoneFetchBids received array as first arg (will use this as the set of div IDs to work on): ', arguments);
            arrDivids = optionalDivIds;
        } else if (typeof optionalDivIds === 'string') {
            // ozoneLog('ozoneFetchBids received string as first arg (will use this as the set of div IDs to work on): ', arguments);
            arrDivids = [optionalDivIds];
        } else {
            ozoneLog('ozoneFetchBids first arg was NOT an array OR a string we will ignore it: ', optionalDivIds);
        }

        if(arrDivids.length === 0) {
            let all = googletag.pubads().getSlots();
            let allDivIds = ozoneWrapper.getDivIdsForSlots(all);
            ozoneLog('ozoneFetchBids will work on all adunits known to googletag ' + JSON.stringify(allDivIds));
        }
        ozoneLog('pushing fn into ozoneWrapper.cmd to call processAdunitsWhichExist');
        // updated code - now passing a list of div IDs to processAdunitsWhichExist
        window.ozoneadunits.cmd.push(function() {
            ozoneLog('*** adUnits have loaded so will call processAdunitsWhichExist now ***', arrDivids);
            ozoneWrapper.processAdunitsWhichExist(arrDivids, callback);
        });

    },

    /**
     * We need to get to the targeting that was present on the googletag objects BEFORE we made any pbjs requests
     * & added additional targeting. Call: targetingManager.getInitialTargetingForDivId('someDivId')
     * @type Object {}
     */
    targetingManager: {
        // store key:[] for each key=divId
        initialTargeting: {},
        /**
         * @param divId
         */
        getInitialTargetingForDivId: function(divId) {
            if(this.initialTargeting.hasOwnProperty(divId)) {
                return this.initialTargeting[divId];
            }
            let slots = ozoneWrapper.getGoogletagSlotsForDivIds([divId]);
            // ozoneLog('getInitialTargetingForDivId got slots: ', slots);
            if(slots.length < 1) {
                ozoneError(`getInitialTargetingForDivId FAILED to find googletag slot for div id ${divId} - will return default {}`);
                return {};
            }
            let slot = slots[0];
            this.initialTargeting[divId] = slot.getTargetingMap();
            // this.initialTargeting[divId] = slot.getTargetingKeys().map(k => {k: slot.getTargeting(k)});
            return this.initialTargeting[divId];
        }
    },

    /**
     * OPTIONALLY receive a set of div IDs, else []
     * Call this ONLY from inside window.ozoneadunits.cmd, so that adunits are definitely in the page
     *
     * @param Array arrDivIds - use all if [] else restrict to those matching these div IDs
     * @param Function callback
     */
    processAdunitsWhichExist: function(arrDivids, callback) {
        if(!arrDivids) arrDivids = [];
        // we are now using the (unique) targeting key value for 'position' to link the googletag slot with our adunits
        let goodAdUnits = []; // list of targeting.position IDs set by googletag setTargeting('position', 1)
        let sm = googletag.pubads().getSlots();
        let goodSlots = [];
        ozoneLog(`processAdunitsWhichExist getSlots found ${sm.length} googletag slots`);
        for(let i = 0; i < sm.length; i++) {
            let property = sm[i];
            let divId = property.getSlotElementId();
            if(arrDivids.length > 0 && arrDivids.indexOf(divId) === -1) {
                // ozoneLog('processAdunitsWhichExist SKIPPING SlotElementId not requested in args: ' + divId);
                continue;
            }
            ozoneLog('processAdunitsWhichExist working on SlotElementId: ' + divId);
            let targetingMap = property.getTargetingMap();
            if(!targetingMap.hasOwnProperty('position')) {
                ozoneError('processAdunitsWhichExist FAILED to find the key: `position` in targetingMap for googletag object: ', JSON.parse(JSON.stringify(property)))
                continue;
            }
            let position = targetingMap.position;
            ozoneLog(`processAdunitsWhichExist found targeting key:value position:${position} in googletag targeting map for SlotElementId ${divId}`);
            let addedCount = this.pushAdunitsForPositionId(goodAdUnits, position, divId, this.targetingManager.getInitialTargetingForDivId(divId));
            ozoneLog(`processAdunitsWhichExist added ${addedCount} adunits to the goodAdunits list for auction`);
            goodSlots.push(property);
        }
        if(goodAdUnits.length < 1) {
            ozoneLog('processAdunitsWhichExist Found NO adunits to send to auction; will do nothing');
            callback(configVars.callbackArg)
            return;
        }
        ozoneLog('processAdunitsWhichExist got list of adunits to send to auction', JSON.parse(JSON.stringify(goodAdUnits)));
        ozoneLog('going to push the auction-call function into pbjs.que');
        pbjs.que.push(function () {
            // new code - we now have to remove existing adunits because we can refresh a subset of them

            // this might be screwing things up when we get loads of ads requests
            // pbjs.removeAdUnit(); // clear out any existing adunits.

            pbjs.addAdUnits(goodAdUnits);
            ozoneLog('About to call requestbids for ', arrDivids, 'pbjs.adUnits is ', JSON.parse(JSON.stringify(pbjs.adUnits)));
            pbjs.requestBids({
                // updated code - we have to send a parameter to sendAdserverRequest now
                bidsBackHandler: function() {
                    ozoneLog('inside bidsbackhandler; about to call setTargetingForGPTAsync');
                    pbjs.setTargetingForGPTAsync(ozoneWrapper.getDivIdsForSlots(goodSlots)); // add adserver targeting from the auction response
                    
                    
                    // allow autovia to go & call refresh();
                    ozoneLog('inside bidsbackhandler; about to call the callback fn', callback);
                    callback(configVars.callbackArg);
                    ozoneLog('inside bidsbackhandler; called the callback fn.');
                },
                adUnits: goodAdUnits /* rather than calling removeAdUnits, do this */
            });
        });
    },

    /**
     * Push adunits whose code matches the `position` into this arr
     *
     * @param arr Array (Passed By Reference; push objects into this array)
     * @param position string
     * @param divId string - set the adunit code to this  in each adunit we push into the array; this is because
     * @param googletagTargeting Object - add this into the targeting we send to auction
     * in our list of adunits our `code` value is not the correct adunit code in the dom, but the googletag `position` value
     *
     * @return int the amount of adunits pushed into the array
     */
    pushAdunitsForPositionId: function(arr, position, divId, googletagTargeting) {
        position = position.toString();
        let count = 0;
        for(let i = 0; i < window.adUnits.length; i++) {
            if(window.adUnits[i].code.toString() === position) {
                // update the adunit code & add the googletag.targeting k/vs as we insert the adunit into the main list:
                let adClone = JSON.parse(JSON.stringify(window.adUnits[i]));
                ozoneLog(`pushAdunitsForPositionId updated adunit.code to: ${divId}`);
                adClone.code = divId;
                Object.assign(adClone.bids[0].params.customData[0].targeting, googletagTargeting);
                arr.push(adClone);
                count++;
            }
        }
        return count;
    },


    // new code - convenience method for getting a list of googletag slot objects from div ids
    getGoogletagSlotsForDivIds: function(arrDivids) {
        ozoneLog('getGoogletagSlotsForDivIds: Going to filter googletag slots by list: ', arrDivids);
        let sm = googletag.pubads().getSlots();
        let ret = sm.filter(element => {
            let slotElementId = element.getSlotElementId();
            if(arrDivids.indexOf(slotElementId) > -1) {
                ozoneLog(`getGoogletagSlotsForDivIds found googletag.slotElementId: ${slotElementId}`);
            } else {
                // ozoneLog(`getGoogletagSlotsForDivIds failed to match for element with slotElementId: ${slotElementId}`);
            }
            return arrDivids.indexOf(slotElementId) > -1;
        });
        ozoneLog('getGoogletagSlotsForDivIds: Going to return: ', ret);
        return ret;
    },
    getDivIdsForSlots: function(arrSlots) {
        let ret = [];
        for(let i = 0; i < arrSlots.length; i++) {
            ret.push(arrSlots[i].getSlotElementId());
        }
        return ret;
    }

}

// simple wrapper function
function ozoneFetchBids() {
    ozoneWrapper.ozoneFetchBids.apply(null, arguments);
}
